@@ -25,6 +25,7 @@ use Fcntl ':flock';
 use Carp 'croak';
 
 use constant DEBUG => 0;
+my %CHILDREN;
 
 our ($bench_message,$bench_time);
 
@@ -38,9 +39,10 @@ sub new {
     my $class       = shift;
     my %args        = @_;
 
-    $args{ReuseAddr}    = 1 unless exists $args{ReuseAddr};
-    $args{LocalPort}  ||= 8101;
-    $args{Listen}     ||= 20;
+    $args{ReuseAddr}          = 1 unless exists $args{ReuseAddr};
+    $args{LocalPort}        ||= 8101;
+    $args{Listen}           ||= 20;
+    $args{PreForkCopies}    ||= 5;
 
     delete $args{LocalPort} if $args{LocalPort} eq 'dynamic'; 
 
@@ -58,11 +60,17 @@ sub new {
     return $self;
 }
 
-sub d           { shift->{daemon}          }
+sub d           { 
+    my $self = shift;
+    my $d    = $self->{daemon};
+    $self->{daemon} = shift if @_;
+    $d;
+}
 sub listen_port { shift->{args}{LocalPort} }
 sub pidfile     { shift->{args}{PidFile}   }
 sub logfile     { shift->{args}{LogFile}   }
 sub user        { shift->{args}{User}      }
+sub prefork_copies { shift->{args}{PreForkCopies}      }
 sub pid         { shift->{pid}             }
 sub config_cache{ shift->{args}{CCacheDir} || File::Spec->catfile(File::Spec->tmpdir,'gbslave_ccache')}
 sub kill        {
@@ -138,46 +146,88 @@ sub run {
     my $quit     = 0;
     my $rotate   = 0;
 
-    $SIG{TERM}   = sub { $quit     = 1 };
-    $SIG{HUP}    = sub { $rotate   = 1 };
+    $SIG{TERM}   = sub { $self->Info('TERM received'); $quit     = 1 };
+    $SIG{HUP}    = sub { $self->Info('HUP received');  $rotate   = 1 };
 
-    my $d     = $self->d;
+    my $d = $self->d;
     $self->Info('GBrowse render slave starting on port ',$self->listen_port);
     $self->do_preload;
 
-    # accept loop in child process
-    while (!$quit) {
+    %CHILDREN = map {$_=>1} $self->prefork($self->prefork_copies);
+
+    # parent sleeps until children need to be "taken care" of
+    if (!%CHILDREN) {
+	$self->request_loop;
+	CORE::exit 0;
+    }
+
+    if (%CHILDREN) { 
+#	$d->close;
+      SLEEP:
+	while (1) {
+	    sleep;
+	    if ($quit) {
+		CORE::kill TERM=>keys %CHILDREN;
+		last SLEEP;
+	    }
+
+	    if ($rotate) {
+		$self->Info("Reopening log file");
+		$self->open_log;
+		$rotate = 0;
+		next SLEEP;
+	    }
+
+	    $self->Info('children remaining = ', join ' ',keys %CHILDREN);
+	    if (keys %CHILDREN < $self->prefork_copies) {
+		my @new_children = $self->prefork($self->prefork_copies - keys %CHILDREN);
+		unless (@new_children) {
+		    $self->request_loop;
+		    CORE::exit 0;
+		}
+		%CHILDREN = (%CHILDREN,map {$_=>1} @new_children);
+		next SLEEP;
+	    }
 
-	if ($rotate) {
-	    $self->Info("Reopening log file");
-	    $self->open_log;
-	    $rotate = 0;
-	    next;
 	}
+	unlink $self->pidfile if $self->pidfile;
+	$self->Info('Normal termination');
+    }
 
-	$self->Debug("Waiting for connection...");
-	my $c     = $d->accept() or next; # accept() is interruptable...
+    CORE::exit 0;
+}
+
+sub prefork {
+    my $self   = shift;
+    my $copies = shift;
+    my @children;
+
+    for (1..$copies) {
 	my $child = fork();
-	$self->Fatal("Couldn't fork: $!") unless defined $child;
+	$DB::inhibit_exit = 0;
+	die "fork() error: $!" unless defined $child;
 	if ($child) {
-	    $self->Info("Forked child PID $child");
-	    $c->close();
+	    $self->Info("preforked pid $child");
+	    push @children,$child;
 	} else {
-	    my $time = time();
-	    Bio::Graphics::Browser2::DataBase->clone_databases();
-	    $self->Debug("clone_database() time: ",time()-$time," s");
-	    $self->process_connection($c);
-	    $self->Debug("process_connection() time: ",time()-$time," s");
-	    $d->close();
-	    $c->close();
-	    my $elapsed = time() - $time;
-	    $self->Debug("Process time $elapsed seconds");
-	    exit 0;
+	    return;
 	}
     }
-    $self->Info('Normal termination');
-    unlink $self->pidfile if $self->pidfile;
-    CORE::exit 0;
+    return @children;
+
+}
+
+sub request_loop {
+    my $self = shift;
+    my $d    = $self->d;
+    $self->preload_databases;
+
+    my $quit = 0;
+    $SIG{TERM}   = sub { $quit     = 1 };
+    while (!$quit) {
+	my $c  = $d->accept() or next; # accept() is interruptable...
+	$self->process_connection($c);
+    }
 }
 
 sub process_connection {
@@ -193,6 +243,7 @@ sub process_connection {
     }
 
     $self->Debug("Finished connection from ",$c->peerhost);
+    $c->close;
 }
 
 
@@ -202,6 +253,7 @@ sub process_request {
 
     $self->Bench('reading request');
 
+    CGI::initialize_globals();
     if ($r->method eq 'GET') {
 	$CGI::Q = CGI->new($r->uri->query);
     } elsif ($r->method eq 'POST') {
@@ -215,24 +267,68 @@ sub process_request {
     }
 
     $self->Debug("process_request(): read ",$r->content_length," bytes");
-
-    $self->Debug("process_request(): setting environment");
-
     $self->Bench('setting environment');
     $self->setup_environment(param('env'));
     my $operation = param('operation') || 'invalid';
 
     $self->Debug("process_request(): operation = $operation");
 
-    my $content   = $operation eq 'render_tracks'   ? $self->render_tracks
-                  : $operation eq 'search_features' ? $self->search_features
-                                                    : 'Invalid request';
+    # make sure databases are already open in parent process
+    $self->Bench('thawing parameters');
+    my $tracks   = thaw param('tracks')     if param('tracks');
+    my $settings = thaw param('settings')   if param('settings');
+    my $dsn      = param('datasource');
+    my $d_name   = param('data_name');
+    my $d_mtime  = param('data_mtime');
+    my $source   = $self->get_datasource($dsn,$d_name,$d_mtime);
+    unless ($source) {
+	$self->Info('sending REQUEST DATASOURCE response');
+	$c->send_response(HTTP::Response->new(403=>"REQUEST DATASOURCE: $d_name"));
+	return;
+    }
+    $source or return (403,"REQUEST DATASOURCE: $d_name");
 
-    my ($status_code,$status_text,$content_type) =
-	 $content =~ /REQUEST DATASOURCE/ ? (403 => $content)
-        :$content !~ /invalid/
-	 ? (200 => 'Ok',          'application/gbrowse-encoded-genome')
-	 : (400 => 'Bad request', 'text/plain');
+    $self->Bench('loading databases');
+    $self->load_databases($tracks,$source) if $tracks;
+    
+    # forking version
+    if ($self->prefork_copies <= 1) {
+	$self->Bench('forking child');
+	my $child = fork();
+	$DB::inhibit_exit = 0;
+	$self->Fatal("Couldn't fork: $!") unless defined $child;
+	if ($child) {
+	    $self->Info("Forked child PID $child");
+	    return;
+	} else {
+	    $self->d->close();
+	    $self->Bench('cloning databases');
+	    Bio::Graphics::Browser2::DataBase->clone_databases();
+	    $self->Bench("operation $operation");
+	    $self->run_operation($c,$operation,$tracks,$source,$settings);
+	    $self->Bench("cleaning up");
+	    exit 0;
+	}
+    }
+    # preforked version
+    else {
+	$self->Bench("operation $operation");
+	$self->run_operation($c,$operation,$tracks,$source,$settings);
+	$self->Bench("cleaning up");
+    }
+}
+
+sub run_operation {
+    my $self      = shift;
+    my ($c,$operation,$tracks,$source,$settings) =  @_;
+
+    my $content   = $operation eq 'render_tracks'   ? $self->render_tracks($tracks,  $source, $settings)
+                  : $operation eq 'search_features' ? $self->search_features($tracks,$source, $settings)
+                                                    : 'invalid request';
+
+    my ($status_code,$status_text,$content_type) = 
+	$content =~ /invalid/i ? (400 => 'Bad request', 'text/plain')
+	                       : (200 => 'Ok',          'application/gbrowse-encoded-genome');
 
     $self->Bench('creating response');
 
@@ -244,12 +340,19 @@ sub process_request {
 
     $self->Bench('sending response');
     $c->send_response($response);
-
-    $self->Bench('cleaning up');
-
     $self->Info("host=",$c->peerhost,"; operation=$operation; response=$status_code; bytes=",length $content);
 }
 
+sub load_databases {
+    my $self   = shift;
+    my ($tracks,$source,$settings) = @_;
+    for my $t (@$tracks) {
+	# this caches databases in memory
+	my $length = defined $settings->{start} ? $settings->{start}-$settings->{stop}+1 : 0;
+	my $db = $source->open_database($t,$length);
+    }
+}
+
 sub Bench {
     my $self = shift;
     my $msg = shift;
@@ -267,6 +370,7 @@ sub get_datasource {
     my $self = shift;
     my ($dsn,$name,$mtime)  = @_;
 
+    $self->Bench('getting datasource');
     mkpath $self->config_cache unless -e $self->config_cache;
 
     if (Storable::read_magic($dsn)) { # this is a storable image
@@ -295,23 +399,10 @@ sub get_datasource {
 
 sub render_tracks {
     my $self = shift;
+    my ($tracks,$datasource,$settings) = @_;
 
-    $self->Debug("render_tracks(): thawing parameters");
-    $self->Bench('thawing parameters');
-    my $tracks	        = thaw param('tracks');
-    my $settings	= thaw param('settings');
     my $language	= thaw param('language');
     my $panel_args      = thaw param('panel_args');
-    my $dsn             = param('datasource');
-    my $d_name          = param('data_name');
-    my $d_mtime         = param('data_mtime');
-
-    $self->Bench('getting datasource');
-
-    my $datasource	= $self->get_datasource($dsn,$d_name,$d_mtime);
-    unless ($datasource) {
-	return "REQUEST DATASOURCE: $d_name";
-    }
 
     $self->do_init($datasource);
     $self->adjust_conf($datasource);
@@ -330,7 +421,7 @@ sub render_tracks {
 	    $db = $datasource->open_database($track) or next;
 	    next if $seenit{$db}++;
 	    ($segment) = $db->segment(-name	=> $settings->{'ref'},
-				      -start=> $settings->{'start'},
+				      -start    => $settings->{'start'},
 				      -stop	=> $settings->{'stop'});
 	    last if $segment;
 	}
@@ -405,11 +496,9 @@ sub render_tracks {
 
 sub search_features {
     my $self = shift;
+    my ($tracks,$datasource,$settings) = @_;
 
     my $searchargs      = thaw param('searchargs');
-    my $tracks	        = thaw param('tracks');
-    my $settings	= thaw param('settings');
-    my $datasource	= thaw param('datasource');
 
     # initialize a region search object
     my $search = Bio::Graphics::Browser2::RegionSearch->new(
@@ -472,7 +561,10 @@ sub become_daemon {
 
     # install signal handler in the master server
     $SIG{CHLD} = sub {
-	while ((my $c = waitpid(-1,WNOHANG))>0) { }
+	while ((my $c = waitpid(-1,WNOHANG))>0) { 
+	    $self->Info("Child pid $c terminated");
+	    delete $CHILDREN{$c};
+	}
     };
     umask(0);
     $ENV{PATH} = '/bin:/sbin:/usr/bin:/usr/sbin';
